@@ -12,8 +12,7 @@ sub call {
 
     $self->validate_env($env);
     my $res = $self->app->($env);
-    $self->validate_res($res);
-    return $res;
+    return $self->validate_res($res);
 }
 
 sub validate_env {
@@ -62,35 +61,43 @@ sub validate_env {
 }
 
 sub validate_res {
-    my ($self, $res) = @_;
+    my ($self, $res, $streaming) = @_;
+
+    my $croak = $streaming ? \&Carp::confess : \&Carp::croak;
 
     unless (ref($res) and ref($res) eq 'ARRAY' || ref($res) eq 'CODE') {
-        Carp::croak('response should be arrayref or coderef');
+        $croak->('response should be arrayref or coderef');
     }
 
-    return unless ref $res eq 'ARRAY';
+    if (ref $res eq 'CODE') {
+        return $self->response_cb($res, sub { $self->validate_res(@_, 1) });
+    }
 
-    unless (@$res == 3) {
-        Carp::croak('response needs to be 3 element array');
+    unless (@$res == 3 || ($streaming && @$res == 2)) {
+        $croak->('response needs to be 3 element array, or 2 element in streaming');
     }
 
     unless ($res->[0] =~ /^\d+$/ && $res->[0] >= 100) {
-        Carp::croak('status code needs to be an integer greater than or equal to 100');
+        $croak->('status code needs to be an integer greater than or equal to 100');
     }
 
     unless (ref $res->[1] eq 'ARRAY') {
-        Carp::croak('Headers needs to be an array ref');
+        $croak->('Headers needs to be an array ref');
     }
 
-    unless (ref $res->[2] eq 'ARRAY' ||
+    # @$res == 2 is only right in psgi.streaming, and it's already checked.
+    unless (@$res == 2 ||
+            ref $res->[2] eq 'ARRAY' ||
             Plack::Util::is_real_fh($res->[2]) ||
             (blessed($res->[2]) && $res->[2]->can('getline'))) {
-        Carp::croak('body should be an array ref or filehandle');
+        $croak->('body should be an array ref or filehandle');
     }
 
     if (ref $res->[2] eq 'ARRAY' && grep utf8::is_utf8($_), @{$res->[2]}) {
-        Carp::croak('body must be bytes and should not contain wide characters (UTF-8 strings).');
+        $croak->('body must be bytes and should not contain wide characters (UTF-8 strings).');
     }
+
+    return $res;
 }
 
 1;
@@ -109,12 +116,15 @@ Plack::Middleware::Lint - Validate request and response
 
 =head1 DESCRIPTION
 
-Plack::Middleware::Lint is a middleware to validate request and
-response environment. Handy to validate missing parameters etc. when
-writing a server or middleware.
+Plack::Middleware::Lint is a middleware component to validate request
+and response environment. You are strongly suggested to use enable
+this middleware when you develop a framework adapter or a new server
+that implements PSGI interface.
 
 =head1 AUTHOR
 
+Tatsuhiko Miyagawa
+
 Tokuhiro Matsuno
 
 =cut
